% PTDIAGRAM Class keeps the p,T-section calculated by TernAPI-C kernel
classdef PTDiagram
    properties
        gsurf;
        convhull;
        regions;
    end
    
    methods
        function obj = PTDiagram(data)
            obj.gsurf = data.gsurf;
            obj.convhull = data.convhull;
            obj.regions = data.diag;
        end
        
        % Finds region(s) with the given name
        % Inputs:
        %   name -- name of the regions
        % Outputs:
        %   indexes -- 1xM matrix with indices of the regions
        function indexes = findRegion(obj, name)
            indexes = [];
            for i = 1:numel(obj.regions)
                if strcmp(obj.regions(i).name, name)
                    indexes = [indexes, i];
                end
            end
        end
        
        % Returns coordinates of tielines
        % Inputs:
        %   reg_id  -- region identifier
        %   coord   -- 'x2' or 'x3'
        % Outputs:
        %   x -- 1xR cell array of 2xN matrices with
        %        xi coordinates of the tielines
        function x = getTielinesCoord(obj, reg_id, coord)
            reg = obj.regions(reg_id);
            x = cell(1, numel(reg));
            xsurf = obj.getSurfXi(coord);
            for i = 1:numel(reg.tielines)
                x{i} = xsurf(reg.tielines{i});
            end
        end
        
        % Returns coordinates of the external boundaries polygons
        % Inputs:
        %   reg_id -- region identifier
        %   coord  -- 'x2' or 'x3'
        function x = getExtBoundPolygonCoord(obj, reg_id, coord)
            reg = obj.regions(reg_id);
            x = cell(1, numel(reg));
            xsurf = obj.getSurfXi(coord);
            for i = 1:numel(reg.extbound_polygon)
                x{i} = xsurf(reg.extbound_polygon{i});
            end
        end
        
        
        function xi = getSurfXi(obj, coord)
            switch coord
                case 'x2'
                    xi = obj.gsurf.x2;
                case 'x3'
                    xi = obj.gsurf.x3;
                otherwise
                    error('Unknown coordinate %s', coord);
            end
        end
    end
    
    methods (Static)
        % Calculates the distances between calculated boundaries
        % made of separate points and the set of experimental points
        % The algorithm:
        % 1) take an experimental point, find the nearest point from 
        %    calculated points (boundary)
        % 2) remember the distance
        % 3) go to the next experimental point
        % Inputs:
        %   xcalc: 1x(D-1) cell array (e.g. {x2calc, x3calc})
        %   xexp: 1x(D-1) cell array (e.g. {x2exp, x3exp})
        % Outputs:
        %   dist 1xM double matrix
        %   nearind: 1xM indices of the nearest points
        function [dist, nearind] = calculateDistance(xcalc, xexp)
            if ~iscell(xcalc)
                error('xcalc must be a cell');
            end            
            if ~iscell(xexp)
                error('xexp must be a cell');
            end
            if numel(xexp) ~= numel(xcalc)
                error('xexp and xcalc sizes must be equal');
            end
            % Estimate the size of output array
            calc_len = numel(xcalc{1});
            exp_len = numel(xexp{1});
            ndim = numel(xexp);            
            for i = 1:numel(xexp)
                if numel(xexp{i}) ~= exp_len
                    error('All xexp vectors must have the same size');
                end
                if numel(xcalc{i}) ~= calc_len
                    error('All xcalc vectors must have the same size');
                end
            end
            % Create empty output arrays
            dist = nan(1, exp_len); % Distances
            nearind = nan(1, exp_len); % Indexes of the nearest points
            % Calculate distances and indices
            for i = 1:exp_len
                d = zeros(1, calc_len);
                for j=1:ndim
                    d = d + (xcalc{j}(:)' - xexp{j}(i)).^2;
                end
                [dist(i), nearind(i)] = min(sqrt(d));
            end
        end        
    end
end